use glam::Vec3;

use crate::plane::{Plane, PlaneIntersection};

#[derive(Default)]
pub struct MaterialId(pub String);

pub struct TextureProjection {
    pub u: Plane,
    pub v: Plane,
}

impl Default for TextureProjection {
    fn default() -> Self {
        Self {
            u: Plane {
                normal: glam::vec3(1.0, 0.0, 0.0),
                offset: 0.0,
            },
            v: Plane {
                normal: glam::vec3(0.0, 1.0, 0.0),
                offset: 0.0,
            },
        }
    }
}

#[derive(Default)]
pub struct BrushPlane {
    pub plane: Plane,
    pub material: MaterialId,
    pub tex_projection: TextureProjection,
}

pub struct Brush {
    pub planes: Vec<BrushPlane>,
}

impl Brush {
    pub fn get_vertices(&self) -> Vec<Vec3> {
        if self.planes.len() < 3 {
            println!("fug");
            return vec![];
        }

        // Test all permutations of brush planes for intersections
        let mut intersections = vec![];
        let mut vs = vec![];
        let len = self.planes.len();
        for i in 0..(len - 2) {
            for j in (i + 1)..(len - 1) {
                for k in (j + 1)..len {
                    if let Some(intersection) = PlaneIntersection::from_planes(
                        self.planes[i].plane,
                        self.planes[j].plane,
                        self.planes[k].plane,
                    ) {
                        intersections.push(intersection)
                    }
                }
            }
        }

        // Validate intersections against other brush planes
        // TODO: No need to check against planes that are part of the intersection
        for intersection in intersections {
            let mut valid = true;
            for bplane in &self.planes {
                if !intersection.in_halfspace_mat(&bplane.plane) {
                    valid = false;
                    break;
                }
            }

            if valid {
                vs.push(intersection.get_intersection_point());
            }
        }

        vs
    }
}
